#include <stdio.h>
#include <stdlib.h>
#include <roxml.h>

#include "ClassesXML.h"

/*!
* \file classesXML.c
* \brief Fichier contenant le code des fonctions d'aquisition des classes.
*/

Classe* Remplir_Classes(FILE* Classes_log, char *filename) /* Remplit le tableau de classes  partir du fichier XML */
{

    fprintf(Classes_log,"I) Chargement du fichier XML - Definition des noeuds\n");

    node_t *rootXML, *Classes;
    unsigned int i=0, j=0, k=0, nb_classes=0, nb_skills=0;

    rootXML = roxml_load_doc(filename);            /* Ouvre le doc XML et cr le noeud de base */
    Classes = roxml_get_chld(rootXML, NULL, 0);    /* Descend d'un niveau dans l'arborescence */
    nb_classes = roxml_get_chld_nb(Classes);    /* Rcupre le nombre de classes entres dans le fichier XML */

    fprintf(Classes_log,"Le fichier contient %d classes\n", nb_classes);

    /*---------------------------------------------------------------------------------------------------*/
    /* Allocation de la mmoire ncssaire et test*/
    /*---------------------------------------------------------------------------------------------------*/

    fprintf(Classes_log,"II) Chargement des classes\n");

    Classe* ListeClasse = NULL;
    ListeClasse = malloc(nb_classes * sizeof(Classe));
    if (ListeClasse == NULL)
    {
        fprintf(Classes_log,"Allocation impossible \n");
        exit(EXIT_FAILURE);
    }
    fprintf(Classes_log,"1) Memoire allouee avec succes \n");
    fprintf(Classes_log,"La memoire allouee fait %d octets\n",nb_classes * sizeof(Classe));

    /*--------------------------------------------------------------------------------------------------*/
    /* Chargement des classes */
    /*--------------------------------------------------------------------------------------------------*/

    for(i=0; i<nb_classes; i++)   /* Boucle de remplissage des classes */
    {
        Classe Classe_en_cours;
        /* Entre dans le noeud de la classe en cours */
        node_t *Classe_node;
        Classe_node = roxml_get_chld(Classes, NULL, i);

        /*----------------------------------------------------------------------------------------------*/
        /* Chargement des caractristiques de la classe en cours */
        /*----------------------------------------------------------------------------------------------*/

        fprintf(Classes_log,"\nChargement des caractristiques de la classe numero %d\n", i+1);


        char *id_text, *name_text, *DESC_text;
        int PV_int, PM_int, MV_int;

        /* Aquisition des stats de la classe en cours */
        id_text = roxml_get_content(roxml_get_attr(Classe_node, "id", 0), NULL, 0, NULL);
        name_text = roxml_get_content(roxml_get_attr(Classe_node, "name", 0), NULL, 0, NULL);
        PV_int = atoi(roxml_get_content(roxml_get_attr(Classe_node, "PV", 0), NULL, 0, NULL));
        PM_int = atoi(roxml_get_content(roxml_get_attr(Classe_node, "PM", 0), NULL, 0, NULL));
        MV_int = atoi(roxml_get_content(roxml_get_attr(Classe_node, "MV", 0), NULL, 0, NULL));
        DESC_text = roxml_get_content(roxml_get_attr(Classe_node, "Desc", 0), NULL, 0, NULL);

        /* Ecriture des stats dans le fichier log */
        fprintf(Classes_log,"Id : %s\n", id_text);
        fprintf(Classes_log,"Nom : %s\n", name_text);
        fprintf(Classes_log,"PV : %d\n", PV_int);
        fprintf(Classes_log,"PM : %d\n", PM_int);
        fprintf(Classes_log,"MV : %d\n", MV_int);
        fprintf(Classes_log,"DESC : %s\n", DESC_text);

        /* Remplissage des caractristiques de la classe en cours */
        Classe_en_cours.PV_max=PV_int;
        Classe_en_cours.mana_max=PM_int;
        Classe_en_cours.points_deplacement_max=MV_int;
        strcpy(Classe_en_cours.nom, id_text);

        /*----------------------------------------------------------------------------------------------*/
        /*Chargement des skins de la classe en cours
        /*----------------------------------------------------------------------------------------------*/

        fprintf(Classes_log,"\nChargement des skins de la classe numero %d\n", i+1);

        node_t* Skins;
        char *id_Skin_text, *default_Skin_text, *selected_Skin_text, *healed_Skin_text, *damaged_Skin_text;

        /* Aquisition des noms de fichiers de textures de la classe en cours */
        Skins = roxml_get_chld(Classe_node, NULL, 1);
        default_Skin_text = roxml_get_content(roxml_get_attr(Skins, "f_default", 0), NULL, 0, NULL);
        selected_Skin_text = roxml_get_content(roxml_get_attr(Skins, "f_selected", 0), NULL, 0, NULL);
        healed_Skin_text = roxml_get_content(roxml_get_attr(Skins, "f_healed", 0), NULL, 0, NULL);
        damaged_Skin_text = roxml_get_content(roxml_get_attr(Skins, "f_damaged", 0), NULL, 0, NULL);

        /* Ecriture des noms de fichiers de textures dans le fichier log */
        fprintf(Classes_log,"\n default : %s", default_Skin_text);
        fprintf(Classes_log,"\n selected : %s", selected_Skin_text);
        fprintf(Classes_log,"\n healed : %s", healed_Skin_text);
        fprintf(Classes_log,"\n damaged : %s\n\n", damaged_Skin_text);

        /* Remplissage des skins de la classe en cours */
        strcpy(Classe_en_cours.skins.charac_default, default_Skin_text);
        strcpy(Classe_en_cours.skins.charac_damaged, damaged_Skin_text);
        strcpy(Classe_en_cours.skins.charac_healed, healed_Skin_text);
        strcpy(Classe_en_cours.skins.charac_selected, selected_Skin_text);

        /*----------------------------------------------------------------------------------------------*/
        /*Chargement des attaques de la classe en cours */
        /*----------------------------------------------------------------------------------------------*/

        fprintf(Classes_log,"\nChargement des attaques de la classe numero %d\n", i+1);

        node_t* Skills;
        Skills = roxml_get_chld(Classe_node, NULL, 0);
        nb_skills = roxml_get_chld_nb(Skills);

        for(j=0 ; j<nb_skills ; j++)
        {

            char *id_Skill_text, *name_Skill_text;
            int RG_int, MN_int, nb_Effects;

            node_t* Skill;
            Skill = roxml_get_chld(Skills, NULL, j);
            nb_Effects = roxml_get_chld_nb(Skill);

            /* Aquisition de l'attaque en cours */
            id_Skill_text = roxml_get_content(roxml_get_attr(Skill, "id", 0), NULL, 0, NULL);
            name_Skill_text = roxml_get_content(roxml_get_attr(Skill, "name", 0), NULL, 0, NULL);
            RG_int = atoi(roxml_get_content(roxml_get_attr(Skill, "RG", 0), NULL, 0, NULL));
            MN_int = atoi(roxml_get_content(roxml_get_attr(Skill, "MN", 0), NULL, 0, NULL));

            /* Ecriture de l'attaque en cours dans le fichier log */
            fprintf(Classes_log,"\nAttaque numero %d :\n", j+1);
            fprintf(Classes_log,"\n ID : %s", id_Skill_text);
            fprintf(Classes_log,"\n Name : %s", name_Skill_text);
            fprintf(Classes_log,"\n RG : %d", RG_int);
            fprintf(Classes_log,"\n MN : %d\n\n", MN_int);

            /* Remplissage de l'attaque en cours */
            strcpy(Classe_en_cours.attaques[j].id, id_Skill_text);
            strcpy(Classe_en_cours.attaques[j].nom, name_Skill_text);
            Classe_en_cours.attaques[j].portee=RG_int;
            Classe_en_cours.attaques[j].mana=MN_int;

            for(k=0 ; k<nb_Effects; k++)
            {
                char *id_Effect_text, *name_Effect_text;
                int TURNS_int, DM_int, DM_DUR_int, HL_int, HL_DUR_int, PARA_int, TRNS_int;

                node_t* Effect;
                Effect = roxml_get_chld(Skill, NULL, k);

                /* Aquisition de l'effet en cours */
                id_Effect_text = roxml_get_content(roxml_get_attr(Effect, "id", 0), NULL, 0, NULL);
                name_Effect_text = roxml_get_content(roxml_get_attr(Effect, "name", 0), NULL, 0, NULL);
                TRNS_int = atoi(roxml_get_content(roxml_get_attr(Effect, "TURNS", 0), NULL, 0, NULL));
                DM_int = atoi(roxml_get_content(roxml_get_attr(Effect, "DM", 0), NULL, 0, NULL));
//                DM_DUR_int = atoi(roxml_get_content(roxml_get_attr(Effect, "DM_DUR", 0), NULL, 0, NULL));
//                HL_int = atoi(roxml_get_content(roxml_get_attr(Effect, "HL", 0), NULL, 0, NULL));
//                HL_DUR_int = atoi(roxml_get_content(roxml_get_attr(Effect, "HL_DUR", 0), NULL, 0, NULL));
                PARA_int = atoi(roxml_get_content(roxml_get_attr(Effect, "PARA", 0), NULL, 0, NULL));

                /* Ecriture de l'effet en cours dans le fichier log */
                fprintf(Classes_log,"\nEffet numero %d :\n", k+1);
                fprintf(Classes_log,"\n ID : %s", id_Effect_text);
                fprintf(Classes_log,"\n Name : %s", name_Effect_text);
                fprintf(Classes_log,"\n Duree : %d", TRNS_int);
                fprintf(Classes_log,"\n DM : : %d", RG_int);
                fprintf(Classes_log,"\n DM_DUR : %d", DM_DUR_int);
                fprintf(Classes_log,"\n HL : : %d", HL_int);
                fprintf(Classes_log,"\n HL_DUR : %d", HL_DUR_int);
                fprintf(Classes_log,"\n PARA : %d\n\n", PARA_int);

                /* Remplissage de l'effet en cours */
                strcpy(Classe_en_cours.attaques[j].effets[k].id, id_Skill_text);
                strcpy(Classe_en_cours.attaques[j].effets[k].nom, name_Skill_text);
                Classe_en_cours.attaques[j].effets[k].nb_tours=TRNS_int;
                Classe_en_cours.attaques[j].effets[k].paralysie=PARA_int;
                Classe_en_cours.attaques[j].effets[k].valeur_degats=DM_int;
                Classe_en_cours.attaques[j].effets[k].valeur_degats_duree=DM_DUR_int;
                Classe_en_cours.attaques[j].effets[k].valeur_soin=HL_int;
                Classe_en_cours.attaques[j].effets[k].valeur_soin_duree=HL_DUR_int;
            }
        }

    /*--------------------------------------------------------------------------------------------------*/
    /* Insertion de la classe dans le tableau */
    /*--------------------------------------------------------------------------------------------------*/
        fprintf(Classes_log,"Insertion de la classe %d dans le tableau\n", i+1);




        /* Ajout de la classe remplie au tableau */
        ListeClasse[i] = Classe_en_cours;
    }

    roxml_free_node(rootXML);
    roxml_free_node(Classes);


    return ListeClasse;
}

